/*
 * ANTs - Fourmis informatiques
 * Copyright (C) 2000 MiKael NAVARRO
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

#define __ANT_C

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <assert.h>

#include "ant.h"

#define MEM 16 /* Ant's memory size */
typedef struct Ant {
  int posx, posy;   /* position */
  int object;       /* freight */
  int memory[MEM];  /* memory */
} _Ant;


/*
 * Constructor.
 */
Ant ant_new(void) {
  int i;
  Ant a = (Ant)malloc(sizeof(_Ant));
  if(!a) { fprintf(stderr, "Can't allocate memory (Ant)\n"); exit(0); }

  /* Inits */
  a->posx = 0;
  a->posy = 0;
  a->object = NONE;
  for(i=0; i<MEM; i++)
    a->memory[i] = NONE;
  
  return a;
}

/*
 * Destructor.
 */
int ant_kill(Ant a) {
  /* Assertion */
  assert(a);

  /* Free memory */
  free(a);
  
  return 1;
}


/*
 * Save objects encounterred in memory.
 * The first entry is the current object encounterred.
 */
int ant_add_memory(Ant a, int object) {
  int i;
  
  /* Assertion */
  assert(a);

  /* Update memory */
  for(i=MEM-1; i>0; i--)
    a->memory[i] = a->memory[i-1];
  a->memory[0] = object;

  return 1;
}


/*
 * Current pos X.
 */
int ant_posx(Ant a) {
  /* Assertion */
  assert(a);

  return a->posx;
}

/*
 * Current pos Y.
 */
int ant_posy(Ant a) {
  /* Assertion */
  assert(a);

  return a->posy;
}

/* 
 * Move on X axis.
 */
int ant_movex(Ant a, int n) {
  /* Assertion */
  assert(a);

  return (a->posx += n);
}

/* 
 * Move on Y axis.
 */
int ant_movey(Ant a, int n) {
  /* Assertion */
  assert(a);

  return (a->posy += n);
}


/*
 * Am I carrying an object?
 */
int ant_is_loaded(Ant a) {
  /* Assertion */
  assert(a);

  return a->object;
}

/* 
 * Load an objet if not already loaded.
 */
int ant_load(Ant a, int object) {
  /* Assertion */
  assert(a);

  if(a->object == NONE)
    a->object = object;
  return a->object;
}

/* 
 * Unload freight.
 */
int ant_unload(Ant a) {
  int object;

  /* Assertion */
  assert(a);

  object = a->object;
  if(object != NONE)
    a->object = NONE;
  return object;
}


/*
 * Compute the occupation rate of current object in memory.
 */
float ant_occupation_rate(Ant a) {
  int occupation=0;
  int i;

  /* Assertion */
  assert(a);

  if(a->memory[0] != NONE) {
    for(i=0; i<MEM; i++)
      if(a->memory[0] == a->memory[i])
	occupation++;
  }
  else if(a->object != NONE) {
    for(i=0; i<MEM; i++)
      if(a->object == a->memory[i])
	occupation++;
  }
  return (float)occupation/MEM;
}
