/*
 * ANTs - Fourmis informatiques
 * Copyright (C) 2000 MiKael NAVARRO
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <assert.h>

#include "nest.h"
#include "ant.h"

#define MAX(a, b) ((a)>(b))?(a):(b)

#define SQ(x) ((x)*(x))
#define K1 1.0
#define K2 1.0

/* Probabilities */
/*
#define P1(a) (SQ(K1/(K1+ant_occupation_rate(a))))
#define P2(a) (SQ(ant_occupation_rate(a)/(K2+ant_occupation_rate(a))))
*/
/*
#define P1(a) (SQ(1-ant_occupation_rate(a)))
#define P2(a) (SQ(ant_occupation_rate(a)))
*/
#define P1(a) (pow(1-ant_occupation_rate(a), 3))
#define P2(a) (pow(ant_occupation_rate(a), 3))
#define P3(a, n) (pow(1-ant_occupation_rate2(a, n), 3))
#define P4(a, n) (pow(ant_occupation_rate2(a, n), 3))


/*
 * Compute the occupation rate according to neighbourhood.
 */
float ant_occupation_rate2(Ant a, Nest n) {
  int occup=0, max=0;
  int object;
  int i, j;

  /* Assertion */
  assert(a && n);
  i = ant_posx(a);
  j = ant_posy(a);
  assert(i >= 0); assert(i < nest_xmax(n));
  assert(j >= 0); assert(j < nest_ymax(n));

  /* Update object */
  if(ant_is_loaded(a)) object = ant_is_loaded(a);
  else if(nest_is_busy(n, i, j)) object = nest_is_busy(n, i, j);
  else object = 0;

  /* Compute occupation rate */
  if(i+1 < nest_xmax(n)) { 
    if(object == nest_is_busy(n, i+1, j)) occup++;
    max++; 
  }
  if(i+1 < nest_xmax(n) && j+1 < nest_ymax(n)) {
    if(object == nest_is_busy(n, i+1, j+1)) occup++; 
    max++; 
  }
  if(j+1 < nest_ymax(n)) {
    if(object == nest_is_busy(n, i, j+1)) occup++; 
    max++; 
  }
  if(i-1 >= 0 && j+1 < nest_ymax(n)) {
    if(object == nest_is_busy(n, i-1, j+1)) occup++; 
    max++; 
  }
  if(i-1 >= 0) {      
    if(object == nest_is_busy(n, i-1, j)) occup++; 
    max++; 
  }
  if(i-1 >= 0 && j-1 >= 0) {
    if(object == nest_is_busy(n, i-1, j-1)) occup++; 
    max++; 
  }
  if(j-1 >= 0) {
    if(object == nest_is_busy(n, i, j-1)) occup++; 
    max++; 
  }
  if(i+1 < nest_xmax(n) && j-1 >= 0) {
    if(object == nest_is_busy(n, i+1, j-1)) occup++; 
    max++; 
  }

  if(i+2 < nest_xmax(n) && j-1 >= 0) {   
    if(object == nest_is_busy(n, i+2, j-1)) occup++; 
    max++; 
  }
  if(i+2 < nest_xmax(n)) {
    if(object == nest_is_busy(n, i+2, j)) occup++; 
    max++; 
  }
  if(i+2 < nest_xmax(n) && j+1 < nest_ymax(n)) {
    if(object == nest_is_busy(n, i+2, j+1)) occup++; 
    max++; 
  }

  if(i+1 < nest_xmax(n) && j+2 < nest_ymax(n)) {
    if(object == nest_is_busy(n, i+1, j+2)) occup++; 
    max++; 
  }
  if(j+2 < nest_ymax(n)) {
    if(object == nest_is_busy(n, i, j+2)) occup++; 
    max++; 
  }
  if(i-1 >= 0 && j+2 < nest_ymax(n)) { 
    if(object == nest_is_busy(n, i-1, j+2)) occup++; 
    max++; 
  }

  if(i-2 >= 0 && j+1 < nest_ymax(n)) {
    if(object == nest_is_busy(n, i-2, j+1)) occup++; 
    max++; 
  }
  if(i-2 >= 0) {
    if(object == nest_is_busy(n, i-2, j)) occup++; 
    max++; 
  }
  if(i-2 >= 0 && j-1 >= 0) {
    if(object == nest_is_busy(n, i-2, j-1)) occup++; 
    max++; 
  }

  if(i-1 >= 0 && j-2 >= 0) {
    if(object == nest_is_busy(n, i-1, j-2)) occup++; 
    max++; 
  }
  if(j-2 >= 0) {
    if(object == nest_is_busy(n, i, j-2)) occup++; 
    max++; 
  }
  if(i+1 < nest_xmax(n) && j-2 >= 0) {
    if(object == nest_is_busy(n, i+1, j-2)) occup++; 
    max++; 
  }

  return (float)occup/(float)max;
}

/*
 * Can I load?
 */
short ant_can_load(Ant a, Nest n) {
  /* Assertion */
  assert(a && n);

  if(!ant_is_loaded(a) && nest_is_busy(n, ant_posx(a), ant_posy(a))) {
    if(((float)rand() / (float)RAND_MAX) 
       < P1(a)) {
      ant_load(a, nest_is_busy(n, ant_posx(a), ant_posy(a)));
      nest_put_down(n, ant_posx(a), ant_posy(a), NONE);
      return 1;
    }
    else return 0;
  }
  else return 0;
}

short ant_can_load2(Ant a, Nest n) {
  /* Assertion */
  assert(a && n);

  if(!ant_is_loaded(a) && nest_is_busy(n, ant_posx(a), ant_posy(a))) {
    if(((float)rand() / (float)RAND_MAX) 
       < P3(a, n)) {
      ant_load(a, nest_is_busy(n, ant_posx(a), ant_posy(a)));
      nest_put_down(n, ant_posx(a), ant_posy(a), NONE);
      return 1;
    }
    else return 0;
  }
  else return 0;
}

/*
 * Can I unload?
 */
short ant_can_unload(Ant a, Nest n) {
  /* Assertion */
  assert(a && n);

  if(!nest_is_busy(n, ant_posx(a), ant_posy(a)) && ant_is_loaded(a)) {
    if(((float)rand() / (float)RAND_MAX) 
       < P2(a)) {
      nest_put_down(n, ant_posx(a), ant_posy(a), ant_unload(a));
      return 1;
    }
    else return 0;
  }
  else return 0;
}

short ant_can_unload2(Ant a, Nest n) {
  /* Assertion */
  assert(a && n);

  if(!nest_is_busy(n, ant_posx(a), ant_posy(a)) && ant_is_loaded(a)) {
    if(((float)rand() / (float)RAND_MAX) 
       < P4(a, n)) {
      nest_put_down(n, ant_posx(a), ant_posy(a), ant_unload(a));
      return 1;
    }
    else return 0;
  }
  else return 0;
}

/*
 * Move the ant on its nest, then load/unload objects.
 */
int ant_move(Ant a, Nest n) {
  int direction = (int) ( ((float)rand() / (float)RAND_MAX) * 8.0);

  /* Assertion */
  assert(a && n);
  assert(ant_posx(a) >= 0); assert(ant_posx(a) < nest_xmax(n));
  assert(ant_posy(a) >= 0); assert(ant_posy(a) < nest_ymax(n));

  /* Desplacement */
  switch(direction) {
  case NORTH:
    ant_movey(a, -1);
    if(ant_posy(a) < 0) ant_movey(a, 2);
    break;
  case EAST:
    ant_movex(a, 1);
    if(ant_posx(a) >= nest_xmax(n)) ant_movex(a, -2);
    break;
  case SOUTH:
    ant_movey(a, 1);
    if(ant_posy(a) >= nest_ymax(n)) ant_movey(a, -2);
    break;
  case WEST:
    ant_movex(a, -1);
    if(ant_posx(a) < 0) ant_movex(a, 2);
    break;
  case NORTH_EAST:
    ant_movey(a, -1);
    if(ant_posy(a) < 0) ant_movey(a, 2);
    ant_movex(a, 1);
    if(ant_posx(a) >= nest_xmax(n)) ant_movex(a, -2);
    break;
  case SOUTH_EAST:
    ant_movey(a, 1);
    if(ant_posy(a) >= nest_ymax(n)) ant_movey(a, -2);
    ant_movex(a, 1);
    if(ant_posx(a) >= nest_xmax(n)) ant_movex(a, -2);
    break;
  case SOUTH_WEST:
    ant_movey(a, 1);
    if(ant_posy(a) >= nest_ymax(n)) ant_movey(a, -2);
    ant_movex(a, -1);
    if(ant_posx(a) < 0) ant_movex(a, 2);
    break;
  case NORTH_WEST:
    ant_movey(a, -1);
    if(ant_posy(a) < 0) ant_movey(a, 2);
    ant_movex(a, -1);
    if(ant_posx(a) < 0) ant_movex(a, 2);
    break;
  default:
    break;
  }

  /* Update memory */
  ant_add_memory(a, nest_is_busy(n, ant_posx(a), ant_posy(a)));
  /* Work */
  ant_can_load(a, n);
  ant_can_unload(a, n); 

  return 1;
}

/*
 * Put pheromone on the track followed by the ant.
 */
int ant_put_down_pheromone(Ant a, Nest n) {
  /* Assertion */
  assert(a && n);

  nest_incr_pheromone(n, ant_posx(a), ant_posy(a));

  return 1;
}

/* 
 * Choose the best cap to follow.
 */
static int ant_best_direction(Ant a, Nest n) {
  int i, best_dir;
  double dir[8]; /* pheromone */
  
  if(ant_posy(a)-1 < 0) 
    dir[NORTH] = -1.0;
  else 
    dir[NORTH] = nest_taux_pheromone(n, ant_posx(a), ant_posy(a)-1);

  if(ant_posy(a)-1 < 0 || ant_posx(a)+1 >= nest_xmax(n))
    dir[NORTH_EAST] = -1.0;
  else
    dir[NORTH_EAST] = nest_taux_pheromone(n, ant_posx(a)+1, ant_posy(a)-1);

  if(ant_posx(a)+1 >= nest_xmax(n))
    dir[EAST] = -1.0;
  else
    dir[EAST] = nest_taux_pheromone(n, ant_posx(a)+1, ant_posy(a));

  if(ant_posy(a)+1 >= nest_ymax(n) || ant_posx(a)+1 >= nest_xmax(n))
    dir[SOUTH_EAST] = -1.0;
  else
    dir[SOUTH_EAST] = nest_taux_pheromone(n, ant_posx(a)+1, ant_posy(a)+1);

  if(ant_posy(a)+1 >= nest_ymax(n))
    dir[SOUTH] = -1.0;
  else
    dir[SOUTH] = nest_taux_pheromone(n, ant_posx(a), ant_posy(a)+1);

  if(ant_posy(a)+1 >= nest_ymax(n) || ant_posx(a)-1 < 0)
    dir[SOUTH_WEST] = -1.0;
  else
    dir[SOUTH_WEST] = nest_taux_pheromone(n, ant_posx(a)-1, ant_posy(a)+1);

  if(ant_posx(a)-1 < 0)
    dir[WEST] = -1.0;
  else
    dir[WEST] = nest_taux_pheromone(n, ant_posx(a)-1, ant_posy(a));

  if(ant_posy(a)-1 < 0 || ant_posx(a)-1 < 0)
    dir[NORTH_WEST] = -1.0;
  else
    dir[NORTH_WEST] = nest_taux_pheromone(n, ant_posx(a)-1, ant_posy(a)-1);

  /* Search the best cap */
  best_dir = -1;
  for(i=0; i<8; i++)
    if(dir[i] > dir[best_dir]) best_dir = i;

  return best_dir;
}

/*
 * Move the ant according to pheromones, then load/unload objects.
 */
int ant_move_following_pheromone(Ant a, Nest n) {
  /* Assertion */
  assert(a && n);
  assert(ant_posx(a) >= 0); assert(ant_posx(a) < nest_xmax(n));
  assert(ant_posy(a) >= 0); assert(ant_posy(a) < nest_ymax(n));

  /* Choose the best cap */
  ant_best_direction(a, n);

  /* Update memory */
  ant_add_memory(a, nest_is_busy(n, ant_posx(a), ant_posy(a)));
  /* Put pheromone */
  ant_put_down_pheromone(a, n);
  /* Work */
  ant_can_load(a, n);
  ant_can_unload(a, n); 

  return 1;
}




