/*
 * ANTs - Fourmis informatiques
 * Copyright (C) 2000 MiKael NAVARRO
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

/*
 * Definition of the Ant struct.,
 * and public interface.
 */

#ifndef _ANT_H
#define _ANT_H

#include "nest.h"

/* Compass card */
#define NORTH 0
#define NORTH_EAST 1
#define EAST 2
#define SOUTH_EAST 3
#define SOUTH 4
#define SOUTH_WEST 5
#define WEST 6
#define NORTH_WEST 7

/* An ant */
typedef struct Ant *Ant; 


/*
 * Constructor.
 */
Ant ant_new(void);

/*
 * Destructor.
 */
int ant_kill(Ant a);


/*
 * Current pos X.
 */
int ant_posx(Ant a) ;

/*
 * Current pos Y.
 */
int ant_posy(Ant a) ;

/* 
 * Move on X axis.
 * Return the new pos X.
 */
int ant_movex(Ant a, int n);

/* 
 * Move on Y axis.
 * Return the new pos Y.
 */
int ant_movey(Ant a, int n);


/*
 * Save objects encounterred in memory.
 * The first entry is the current object encounterred.
 */
int ant_add_memory(Ant a, int object);

/*
 * Compute the occupation rate of current object in memory.
 */
float ant_occupation_rate(Ant a);

/*
 * Compute the occupation rate according to neighbourhood.
 */
float ant_occupation_rate2(Ant a, Nest n);

/*

 * Am I carrying an object?
 * Return the object type or NONE.
 */
int ant_is_loaded(Ant a);

/* 
 * Load an objet if not already loaded.
 * Return the object loaded.
 */
int ant_load(Ant a, int object);

/* 
 * Unload freight.
 */
int ant_unload(Ant a);

/*
 * Can I load?
 * Return 1 if object loaded, else 0.
 */
short ant_can_load(Ant a, Nest n);
short ant_can_load2(Ant a, Nest n);

/*
 * Can I unload?
 * Return 1 if object unloaded, else 0.
 */
short ant_can_unload(Ant a, Nest n);
short ant_can_unload2(Ant a, Nest n);


/*
 * Move the ant on its nest, then load/unload objects.
 */
int ant_move(Ant a, Nest n);


/*
 * Put pheromone on the track followed by the ant.
 */
int ant_put_down_pheromone(Ant a, Nest n);

/*
 * Move the ant according to pheromones, then load/unload objects.
 */
int ant_move_following_pheromone(Ant a, Nest n);


#endif /* _ANT_H */
