/*
 * ANTs - Fourmis informatiques
 * Copyright (C) 2000 MiKael NAVARRO
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

/*
 * Definition of the Nest struct.,
 * and public interface.
 */

#ifndef _NEST_H
#define _NEST_H

/* Objects types */
#define NONE 0
#define EGG 1
#define LARVA 2
#define COCOON 3

/* Max pheromone rate */
#define MAX_PHEROMONE 10

/* The nest */
typedef struct Nest *Nest;


/*
 * Constructor.
 */
Nest nest_new(int x, int y);

/*
 * Destructor.
 */
int nest_destroy(Nest n);


/* 
 * SizeX (begin at 0).
 */
int nest_xmax(Nest n);

/*
 * SizeY (begin at 0).
 */
int nest_ymax(Nest n);


/*
 * Object encounterred? Or, pos (x,y) is busy?
 * Return the object type encounterred.
 */
int nest_is_busy(Nest n, int x, int y);

/*
 * Update nest (object) in pos (x, y).
 * Return the object or 0 if pos already busy.
 */
int nest_put_down(Nest n, int x, int y, int object);


/*
 * Return pheromone rate on pos (x,y).
 */
double nest_taux_pheromone(Nest n, int x, int y);

/*
 * Put pheromones at pos (x, y).
 */
int nest_incr_pheromone(Nest n, int x, int y);

/*
 * Pheromones are volatilizable...
 */
int nest_pheromone_evaporation(Nest n);


/* 
 * Display nest.
 */
int nest_display(Nest n);


#endif /* _NEST_H */
