/*
 * ANTs - Fourmis informatiques.
 * Copyright (C) 2000 MiKael NAVARRO
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>

#include <time.h>

#ifdef UNISTD
#include <unistd.h>
#endif
#include <sys/utsname.h>
#include <getopt.h> /* part of GNU C library */

#include "nest.h"
#include "ant.h"

/* 
 * Print help/usage.
 */
static int print_usage(void) {
  printf("\nUsage: triants [-seed n] [--help] [--version]\n"
	 "ANTs - Fourmis informatiques.\n\n"
	 "  Options:\n"
	 "  --seed n       seed for a new sequence of pseudo-random integers\n"
	 "  --nestw n      nest width\n"
	 "  --nesth n      nest height\n"
	 "  --ants n       ants number\n"
	 "  --version      output version information and exit\n"
	 "  --help         display this message and exit\n"
	 "\nReport bugs to klnavarro@free.fr or klnavarro@voila.fr\n");
  return 0;
}

/* 
 * Sort.
 */
int main(int argc, char **argv) {
  Ant *a;
  int nbants=20;
  Nest n;
  int xmax=80, ymax=20;
  time_t t0;
  int i, j;

  int c, opt_ind=0;
  static struct option long_options[] = {
    {"seed", 1, 0, 's'},
    {"nestw", 1, 0, 'x'},
    {"nesth", 1, 0, 'y'},
    {"ants", 1, 0, 'a'},
    {"help", 0, 0, 'h'},
    {"version", 0, 0, 'r'},
    {0, 0, 0, 0}
  };

  /* Command line options */
  while((c=getopt_long(argc, argv, "rhs:x:y:a:", long_options, &opt_ind))!=EOF) {
    switch(c) {
    case 's':
      srand(atoi(optarg));
      break;
    case 'x':
      xmax = atoi(optarg);
      break;
    case 'y':
      ymax = atoi(optarg);
      break;
    case 'a':
      nbants = atoi(optarg);
      break;
    case 'h':
      print_usage(); return 0;
    case 'r':
      printf("%s %.1f\n", argv[0], VERSION); return 0;
    default:
      break;
    }
  }
  /* Others args. */
  if(optind < argc) {
  }
  else {
    /* Too few args. */
  }

  /* Init nest */
  n = nest_new(xmax, ymax);
  printf("NEST : (%dx%d) %d ants, (0 step, 0 seconds)\n", xmax, ymax, nbants); 
  for(j=0; j<xmax; j++) printf("="); printf("\n");
  nest_display(n);
  
  /* Lay ants */
  a = (Ant*)malloc(sizeof(Ant) * nbants);
  for(i=0; i<nbants; i++)
    a[i] = ant_new();

  /* Ants at work */
  t0 = time(NULL);
  i = 1;
  while(1) {
    for(j=0; j<nbants; j++)
      ant_move(a[j], n);
    i++;
    if(i%1000 == 0) {
      printf("\nNEST : (%dx%d) %d ants, (%d step, %d seconds)\n", 
	     xmax, ymax, nbants, i, (int)difftime(time(NULL), t0)); 
      for(j=0; j<xmax; j++) printf("="); printf("\n");
      nest_display(n);
    }
  }

  /* Free memory */
  for(i=0; i<nbants; i++)
    ant_kill(a[i]);
  free(a);
  nest_destroy(n);
  return 0;
}
